"""
A well documented and typed API wrapper for https://hentaihaven.xxx.

Documentation: https://jokelbaf.github.io/hhaven

Source Code: https://github.com/jokelbaf/hhaven
"""

from . import models, exceptions
from .client import *

__version__ = "0.1.4"
